<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace FiloBlu\Rma\Model\Service;

use FiloBlu\Rma\Helper\SerializerHelper;
use Magento\Framework\Api\SimpleDataObjectConverter;
use Magento\Framework\App\Config\ScopeConfigInterface;
use FiloBlu\Rma\Api\RmaAttributesManagementInterface;
use Magento\Customer\Model\AttributeMetadataConverter;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Customer\Model\AttributeMetadataDataProvider;
use Magento\Eav\Model\Entity\Attribute\AbstractAttribute;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Catalog\Model\ResourceModel\Product;
use FiloBlu\Rma\Helper\RmaHelper;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;
use Magento\Framework\App\ObjectManager;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use Magento\Sales\Model\Order\ItemRepository;
use Magento\Catalog\Model\ProductRepository;
use Magento\ConfigurableProduct\Api\LinkManagementInterface;
use FiloBlu\Rma\Logger\Logger;
use Magento\Framework\Module\Manager;
use Magento\CatalogInventory\Api\StockStateInterface;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;

/**
 * Class RmaAttributesManagement
 */
class RmaAttributesManagement implements RmaAttributesManagementInterface
{

    const STOCK_MIN_QTY = 'filoblu_disablerma_section/rma_group/rma_minimum_qty';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var SerializerHelper
     */
    protected $serializer;

    /**
     * List of item dto methods
     *
     * @var array
     */
    protected $dataObjectMethods = [];

    /**
     * Attribute metadata provider
     *
     * @var AttributeMetadataDataProvider
     */
    protected $metadataDataProvider;

    /**
     * Attribute metadata converter
     *
     * @var AttributeMetadataConverter
     */
    protected $metadataConverter;

    /**
     * @var ChangeSizeAttributeHelper
     */
    protected $changeSizeAttributeHelper;

    /**
     * @var ItemRepository
     */
    protected $itemRepository;

    /**
     * @var ProductRepository
     */
    protected $productRepository;

    /**
     * @var LinkManagementInterface
     */
    protected $linkManagement;

    /**
     * @var Product
     */
    protected $resourceModel;

    /**
     * @var Configurable
     */
    protected $configurableResourceModel;

    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var ObjectManager
     */
    protected $objectManager;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var Manager
     */
    protected $manager;

    /**
     * @var StockStateInterface
     */
    protected $stockItem;

    /**
     * Constructor
     *
     * @param AttributeMetadataDataProvider $metadataDataProvider
     * @param AttributeMetadataConverter $metadataConverter
     */
    public function __construct(
        AttributeMetadataDataProvider $metadataDataProvider,
        AttributeMetadataConverter $metadataConverter,
        ScopeConfigInterface $scopeConfig,
        SerializerHelper $serializer,
        StoreManagerInterface $storeManager,
        ChangeSizeAttributeHelper $changeSizeAttributeHelper,
        ItemRepository $itemRepository,
        ProductRepository $productRepository,
        LinkManagementInterface $linkManagement,
        Product $resourceModel,
        Configurable $configurableResourceModel,
        CollectionFactory $collectionFactory,
        Logger $logger,
        Manager $manager,
        StockStateInterface $stockItem
    ) {
        $this->metadataDataProvider = $metadataDataProvider;
        $this->metadataConverter = $metadataConverter;
        $this->scopeConfig = $scopeConfig;
        $this->serializer = $serializer;
        $this->changeSizeAttributeHelper =$changeSizeAttributeHelper;
        $this->itemRepository =$itemRepository;
        $this->productRepository = $productRepository;
        $this->linkManagement = $linkManagement;
        $this->resourceModel = $resourceModel;
        $this->configurableResourceModel = $configurableResourceModel;
        $this->collectionFactory = $collectionFactory;
        $this->objectManager = ObjectManager::getInstance();
        $this->logger = $logger;
        $this->storeManager = $storeManager;
        $this->manager = $manager;
        $this->stockItem = $stockItem;
    }

    /**
     * Retrieve all attributes filtered by form code
     *
     * @param string $formCode
     * @return \Magento\Customer\Api\Data\AttributeMetadataInterface[]
     */
    public function getAttributes($formCode)
    {
        $attributes = [];
        $attributesFormCollection = $this->metadataDataProvider->loadAttributesCollection(
            self::ENTITY_TYPE,
            $formCode
        );
        foreach ($attributesFormCollection as $attribute) {
            /** @var $attribute \Magento\Customer\Model\Attribute */
            $attributes[$attribute->getAttributeCode()] = $this->metadataConverter->createMetadataAttribute($attribute);
        }

        return $attributes;
    }

    /**
     * Retrieve attribute metadata
     *
     * @param string $attributeCode
     * @return \Magento\Customer\Api\Data\AttributeMetadataInterface
     * @throws NoSuchEntityException
     */
    public function getAttributeMetadata($attributeCode)
    {
        /** @var AbstractAttribute $attribute */
        $attribute = $this->metadataDataProvider->getAttribute(self::ENTITY_TYPE, $attributeCode);
        if ($attribute && $attribute->getIsVisible() == 1) {
            $attributeMetadata = $this->metadataConverter->createMetadataAttribute($attribute);
            return $attributeMetadata;
        } else {
            throw new NoSuchEntityException(
                __(
                    'No such entity with %1 = %2, %3 = %4',
                    'entityType',
                    self::ENTITY_TYPE,
                    'attributeCode',
                    $attributeCode
                )
            );
        }
    }

    /**
     * Get all attribute metadata
     *
     * @return \Magento\Customer\Api\Data\AttributeMetadataInterface[]
     */
    public function getAllAttributesMetadata()
    {
        /** @var AbstractAttribute[] $attribute */
        $attributeCodes = $this->metadataDataProvider->getAllAttributeCodes(
            self::ENTITY_TYPE,
            self::ATTRIBUTE_SET_ID
        );

        $attributesMetadata = [];
        foreach ($attributeCodes as $attributeCode) {
            try {
                $attribute = $this->getAttributeMetadata($attributeCode);
                if ($attributeCode == 'reason') {

                    $configData = $this->scopeConfig->getValue('sales/rma_reason/reason') ?? '';

                    if ($configData != '') {
                        $rma_reason_options = array();
                        $mapping = $this->serializer->unserialize($configData);
                        $allowedReason = [];
                        if (count($mapping) > 0) {
                            foreach ($attribute->getOptions() as $option){
                                $visible = true;
                                foreach ($mapping as $map){
                                    if($map['brand_reason'] == $option->getValue()){
                                        if($map['is_visible'] != 1){
                                            $visible = false;
                                        }
                                    }
                                }
                                if($visible){
                                    $allowedReason[] = $option;
                                }
                            }
                        }
                        $attribute->setOptions($allowedReason);
                    }
                }
                $attributesMetadata[] = $attribute;
            } catch (NoSuchEntityException $e) {
                // If no such entity, skip
            }
        }

        return $attributesMetadata;
    }

    /**
     *  Get custom attribute metadata for the given Data object's attribute set
     *
     * @param string|null $dataObjectClassName Data object class name
     * @return \Magento\Framework\Api\MetadataObjectInterface[]
     */
    public function getCustomAttributesMetadata($dataObjectClassName = self::DATA_OBJECT_CLASS_NAME)
    {
        $customAttributes = [];
        if (!$this->dataObjectMethods) {
            $this->dataObjectMethods = array_flip(get_class_methods($dataObjectClassName));
        }
        foreach ($this->getAllAttributesMetadata() as $attributeMetadata) {
            $attributeCode = $attributeMetadata->getAttributeCode();
            $camelCaseKey = SimpleDataObjectConverter::snakeCaseToUpperCamelCase($attributeCode);
            $isDataObjectMethod = isset($this->dataObjectMethods['get' . $camelCaseKey])
                || isset($this->dataObjectMethods['is' . $camelCaseKey]);

            /** Even though disable_auto_group_change is system attribute, it should be available to the clients */
            if (!$isDataObjectMethod
                && (!$attributeMetadata->isSystem() || $attributeCode == 'disable_auto_group_change')
            ) {
                $customAttributes[] = $attributeMetadata;
            }
        }

        return $customAttributes;
    }

    /**
     * @param int $itemId
     * @param int $qty
     * @return \Magento\Customer\Api\Data\AttributeMetadataInterface[]
     */
    public function getAllAttributesWithAlternativesMetadata(int $itemId,int $qty)
    {
        /** @var AbstractAttribute[] $attribute */
        $attributeCodes = $this->metadataDataProvider->getAllAttributeCodes(
            self::ENTITY_TYPE,
            self::ATTRIBUTE_SET_ID
        );

        $attributesMetadata = [];
        foreach ($attributeCodes as $attributeCode) {
            try {
                $attribute = $this->getAttributeMetadata($attributeCode);
                if ($attributeCode == 'reason') {

                    $configData = $this->scopeConfig->getValue('sales/rma_reason/reason') ?? '';

                    if ($configData != '') {
                        $rma_reason_options = array();
                        $mapping = $this->serializer->unserialize($configData);
                        $allowedReason = [];
                        if (count($mapping) > 0) {
                            foreach ($attribute->getOptions() as $option){
                                $visible = true;
                                foreach ($mapping as $map){
                                    if($map['brand_reason'] == $option->getValue()){
                                        if($map['is_visible'] != 1){
                                            $visible = false;
                                        }
                                    }
                                }
                                if($visible){
                                    $allowedReason[] = $option;
                                }
                            }
                        }
                        $attribute->setOptions($allowedReason);
                    }
                }
                elseif($attributeCode == 'resolution'){
                    try {
                        $storeId = $this->storeManager->getStore()->getId();
                        $size = [];
                        $allowedResolution = [];
                        foreach ($attribute->getOptions() as $option) {
                            if(!(int)$option->getValue()) continue;
                            if ((int)$option->getValue() && (int)$option->getValue() == $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId()) {
                                if ($qty == 0) {
                                    $qty = 1;
                                }
                                $item = $this->itemRepository->get($itemId);
                                $product = $this->productRepository->get($item->getSku(), false, $storeId);
                                $parent_products = $this->configurableResourceModel->getParentIdsByChild($product->getId());
                                if (!$parent_products) {
                                    continue;
                                }
                                $parents_sku = $this->resourceModel->getProductsSku($parent_products);
                                $childProducts = $this->linkManagement->getChildren($parents_sku[0]['sku']);

                                foreach ($childProducts as $child) {
                                    //if ($this->isSaleable($child, $storeId)) {
                                        if ($child->getId() === null) {
                                            $child->setId($child->getIdBySku($child->getSku()));
                                        }
                                        if ($this->getProductQtyById($child->getId(), $storeId) >= $this->getMinQty() + $qty && $product->getId() != $child->getId()) {
                                            $size[] = ["value" => $child->getId(), "text" => $child->getResource()->getAttribute('size')->getFrontend()->getValue($child), "qty" =>  $this->getProductQtyById($child->getId(), $storeId)];
                                        }
                                    //}
                                }
                                if (count($size) > 0) {
                                    $option->setData('options',$size);
                                    $allowedResolution[] = $option;
                                }
                            }
                            else{
                                $allowedResolution[] = $option;
                            }
                        }
                        $attribute->setOptions($allowedResolution);
                    } catch (\Exception $e) {
                        $this->logger->critical($e->getMessage());
                    }
                }
                $attributesMetadata[] = $attribute;
            } catch (NoSuchEntityException $e) {
                $this->logger->critical($e->getMessage());
            }
        }

        return $attributesMetadata;
    }

    /**
     * @param $product
     * @param int $storeId
     * @return bool
     */
    public function isSaleable($product, int $storeId)
    {
        try{
            $websiteId = $this->storeManager->getStore($storeId)->getWebsiteId();
            $products = $this->collectionFactory->create()
                ->setStoreId($storeId)
                ->addPriceData($this->customerSession->getCustomerGroupId(),$websiteId)
                ->addAttributeToFilter('sku',['eq'=>$product->getSku()])
                ->getItems();
            return $products ? true : false;
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return false;
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
            return false;
        }
    }


    /**
     * @param $productId
     * @param $storeId
     * @return float|int
     */
    public function getProductQtyById($productId,$storeId)
    {
        try{
            if(!interface_exists('Magento\InventorySalesApi\Api\GetProductSalableQtyInterface') || !$this->manager->isEnabled('Magento_InventorySalesApi')){
                return $this->stockItem->getStockQty($productId);
            }
            $websiteCode = $this->storeManager->getWebsite($this->storeManager->getStore($storeId)->getWebsiteId())->getCode();
            $getProductSalableQty = $this->objectManager->create(\Magento\InventorySalesApi\Api\GetProductSalableQtyInterface::class);
            $stockResolver = $this->objectManager->create(\Magento\InventorySalesApi\Api\StockResolverInterface::class);
            $stockId = $stockResolver->execute(SalesChannelInterface::TYPE_WEBSITE, $websiteCode)->getStockId();
            $product = $this->productRepository->getById($productId);
            return (int)$getProductSalableQty->execute($product->getSku(), $stockId);
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return 0;
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
            return 0;
        }

    }

    /**
     * @return mixed
     */
    public function getMinQty()
    {
        return (int) $this->scopeConfig->getValue(self::STOCK_MIN_QTY);
    }
}
